@@ -17,7 +17,6 @@
  */
 #include "pqxx/compiler.h"
 
-#include <cassert>	// DEBUG CODE
 #include <stdexcept>
 
 #include "pqxx/except"
@@ -29,16 +28,17 @@ using namespace PGSTD;
 
 pqxx::result &pqxx::result::operator=(const pqxx::result &Other) throw ()
 {
-  if (Other.m_Result != m_Result)
+  if (&Other != this)
   {
     LoseRef();
     MakeRef(Other);
   }
+
   return *this;
 }
 
 
-pqxx::result &pqxx::result::operator=(PGresult *Other)
+pqxx::result &pqxx::result::operator=(PGresult *Other) throw ()
 {
   if (Other != m_Result)
   {
@@ -122,8 +122,6 @@ string pqxx::result::StatusError() const
 
 void pqxx::result::MakeRef(PGresult *Other) throw ()
 {
-  assert(m_l == this);
-  assert(m_r == this);
   m_Result = Other;
 }
 
@@ -132,6 +130,7 @@ void pqxx::result::MakeRef(const pqxx::result &Other) throw ()
 {
   m_l = &Other;
   m_r = Other.m_r;
+
   m_l->m_r = m_r->m_l = this;
   m_Result = Other.m_Result;
 }
@@ -139,11 +138,7 @@ void pqxx::result::MakeRef(const pqxx::result &Other) throw ()
 
 void pqxx::result::LoseRef() throw ()
 {
-  if (m_l == this)
-  {
-    assert(m_r == this);
-    if (m_Result) PQclear(m_Result);
-  }
+  if ((m_l == this) && m_Result) PQclear(m_Result);
   m_Result = 0;
   m_l->m_r = m_r;
   m_r->m_l = m_l;
