@@ -55,6 +55,7 @@ int main(int argc, char *argv[])
 
     tablewriter W(T, TableName);
 
+    // TODO: Move this stuff to a file!
     const char *const CData[][2] =
     {
       {   "71", "jtv" },
@@ -66,6 +67,7 @@ int main(int argc, char *argv[])
       { "2002", "libpqxx" },
       { "1989", "Ode an die Freiheit" },
       { "2001", "New millennium" },
+      { "1974", "" },
       {   "97", "Asian crisis" },
       { "2001", "A Space Odyssey" },
       {0,0}
